$fn=360;
DEBUG=false;
SIZE=60;
HEIGHT=4.6;
THICKNESS=3;
PART=0.7;
STRETCH_LEN=40;
TIP_LEN=0.05;
TIP_BEND=20;
RAKE_DEPTH=1;
RAKE_WIDTH=1;
RAKE_STRETCH=1;
RAKE_CHAMPFER=0.5;

module partial_ring(part, radius, thickness, height) {
    rotate(180-180*part, [0, 0, 1])
        rotate_extrude(angle=360*part)
            translate([radius, 0])
                square([thickness, height], center=true);
}

module headband(debug=DEBUG, size=SIZE, height=HEIGHT, thickness=THICKNESS, part=PART, stretch_len=STRETCH_LEN, tip_len=TIP_LEN, tip_bend=TIP_BEND, rake_depth=RAKE_DEPTH, rake_width=RAKE_WIDTH, rake_stretch=RAKE_STRETCH, rake_champfer=RAKE_CHAMPFER) {
    union() {
        color("purple") partial_ring(part, radius=size, thickness=thickness, height=height);
        rotate( 90-180*part, [0, 0, 1]) translate([stretch_len/2,  size, 0]) {
            color("orange") cube([stretch_len, thickness, height], center=true);
            color("purple") translate([stretch_len/2, tip_bend, 0])
                rotate(90+180*tip_len, [0, 0, 1])
                    partial_ring(
                        part=tip_len,
                        radius=tip_bend,
                        thickness=thickness,
                        height=height
                    );
        }
        rotate(-90+180*part, [0, 0, 1]) translate([stretch_len/2, -size, 0]) {
            color("orange") cube([stretch_len, thickness, height], center=true);
            color("purple") scale([1, -1, 1]) translate([stretch_len/2, tip_bend, 0])
                rotate(90+180*tip_len, [0, 0, 1])
                    partial_ring(
                        part=tip_len,
                        radius=tip_bend,
                        thickness=thickness,
                        height=height
                    );
        }
        for (i=[-size/2:size/2]) {
            rotate(i*2-rake_width/2, [0, 0, 1]) intersection() {
                scale([1, 1, rake_stretch/(rake_depth/height)])
                    translate([0, rake_width/2, 0]) rotate_extrude(angle=rake_width)
                        translate([rake_depth/2-size+thickness/2, 0, 0])
                            scale([1, 0.5, 1])
                                circle(r=rake_depth, $fn=3);
                translate([rake_depth/2-size+thickness/2-rake_champfer, 0, 0])
                    color("red")
                        cube([2*rake_depth, rake_width, height], center=true);
            }
        };
    };
}

module bunnyear_headband(debug=DEBUG, size=SIZE, height=HEIGHT, thickness=THICKNESS, part=PART, stretch_len=STRETCH_LEN, tip_len=TIP_LEN, tip_bend=TIP_BEND, ear_scale=0.5, ear_bend_factor=0.2, ear_stretch_factor=10, ear_angle=30) {
  catear_headband(
    debug=debug,
    size=size,
    height=height,
    thickness=thickness,
    part=part,
    stretch_len=stretch_len,
    tip_len=tip_len,
    tip_bend=tip_bend,
    ear_scale=ear_scale,
    ear_bend_factor=ear_bend_factor,
    ear_stretch_factor=ear_stretch_factor,
    ear_angle=ear_angle);
}

module foxear_headband(debug=DEBUG, size=SIZE, height=HEIGHT, thickness=THICKNESS, part=PART, stretch_len=STRETCH_LEN, tip_len=TIP_LEN, tip_bend=TIP_BEND, ear_scale=1.2, ear_bend_factor=0.01, ear_stretch_factor=1.2, ear_angle=30) {
  catear_headband(
    debug=debug,
    size=size,
    height=height,
    thickness=thickness,
    part=part,
    stretch_len=stretch_len,
    tip_len=tip_len,
    tip_bend=tip_bend,
    ear_scale=ear_scale,
    ear_bend_factor=ear_bend_factor,
    ear_stretch_factor=ear_stretch_factor,
    ear_angle=ear_angle);
}
